#!/usr/bin/perl
#
# @(#)ToolLib       1998-04-14
#
# Collection of useful tools
#
# G. Gromann, SAP AG
#
#
#    ========== licence begin  GPL
#    Copyright (C) 2001 SAP AG
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of the GNU General Public License
#    as published by the Free Software Foundation; either version 2
#    of the License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#    ========== licence end
#

package ToolLib;
require 5.000;
use Env;
@EXPORT_OK = qw(CreateBuildNrModule GetBuildNrModule SetBuildNrModule ToolTrace SrcCopy $TraceFile );
use Carp;
use FileHandle;
use SDBM_File;

#require "sutvars$ENV{TOOLEXT}";
do "$TOOL/bin/sutvars$TOOLEXT";

my $BuildNrFunction;
if ( $RELVER eq "R62" )
{
    $BuildNrFunction="s98linknumber";
}
else
{
    $BuildNrFunction="s100buildnumber";
}
return( 1 );


###############################################################

sub CreateBuildNrModule
{
    if ( $ENV{RELVER} ne "R61" && $ENV{RELVER} ne "TOOL" && $ENV{RELVER} ne "RTOOL")
    {
        return() if ( ! $BuildNrFunction );
        local $BuildObj = GetBuildNrModule().".o";
        return( $BuildObj ) if ( -f $BuildObj );
        local @Flags = @_;
        local $BuildModule = GetBuildNrModule().".c";
        local $fd = new FileHandle;
        if ( ! $fd->open($BuildModule, "w") )
        {
            die "Can't open $BuildModule (output) :$!\n";
        }
        else
        {
			# PTS 1105044
			if ( $RELVER ge "R72" )
			{
				$fd->print("#define RESOURCE_FILE\n");
				$fd->print("#include \"gsp100.h\"\n");
				$fd->print("char * $BuildNrFunction(){return \"$ENV{BUILDSTRING}\";}\n");
				# $fd->print("char * $BuildNrFunction(){return BUILD_PREFIX_STR_SP100\"-$ENV{BUILDSTRING}\";}\n");
				$fd->print("char * ${BuildNrFunction}Ex(unsigned char * buildpraefix, unsigned long * buildnr)\n");
				$fd->print("{\n\tif ( buildpraefix ) * buildpraefix = BUILD_PREFIX_SP100;\n");
				$fd->print("\tif ( buildnr ) * buildnr = $ENV{BUILDNUMBER};\n");
				# $fd->print("\treturn BUILD_PREFIX_STR_SP100\"-$ENV{BUILDSTRING}\";\n}\n");
				$fd->print("\treturn \"$ENV{BUILDSTRING}\";\n}\n");
			}
			else
			{
				if (not defined($ENV{BUILDPRAEFIX})) { $ENV{BUILDPRAEFIX} = 0 }
				$fd->print("char * $BuildNrFunction(){return \"$ENV{BUILDSTRING}\";}\n");
				$fd->print("char * ${BuildNrFunction}Ex(unsigned char * buildpraefix, unsigned long * buildnr)\n");
				$fd->print("{\n\tif ( buildpraefix ) * buildpraefix = $ENV{BUILDPRAEFIX};\n");
				$fd->print("\tif ( buildnr ) * buildnr = $ENV{BUILDNUMBER};\n");
				$fd->print("\treturn \"$ENV{BUILDSTRING}\";\n}\n");
			}

            do "$TOOLVARS";
			if ( $@ ) { print "$@"; exit }

            if ($ENV{RELVER} eq "RTOOL")
			{
				$REL_STRING = "Tool";
			} else {
				($FOO) = $RELVER =~ /^.(.)/i;
				($BAR) = $RELVER =~ /^..(.)/i;
				$REL_STRING = "Rel. ".$FOO.".".$BAR.".".$ENV{CORRECTION_LEVEL};
			}
			# PTS 1105044
			if ( $RELVER ge "R72" )
			{
				$fd->print("$MF_ID \"@(#)$REL_STRING Build: \"BUILD_PREFIX_STR_SP100\"-$ENV{BUILDSTRING}\" $ID_END\n");
			}
			else
			{
				$fd->print("$MF_ID \"@(#)$REL_STRING Build: $ENV{BUILDSTRING} \"$ID_END\n");
			}
			require OSRTE;
			import OSRTE;
			$fd->print ("$OSRTE_OS_ID \"@(#)Operating System: $OSRTE{'OS'}\"$OSRTE_ID_END\n");
			$fd->print ("$OSRTE_ARCH_ID \"@(#)Architecture: $OSRTE{'ARCH'}\"$OSRTE_ID_END\n");
			$fd->print ("$OSRTE_CC_ID \"@(#)C Compiler: $OSRTE{'CC'}\"$OSRTE_ID_END\n");
			$fd->print ("$OSRTE_CPP_ID \"@(#)C++ Compiler: $OSRTE{'CPLUSPLUS'}\"$OSRTE_ID_END\n");
			$fd->print("void ShowProduction$BuildNrFunction()\n");
			$fd->print("{\n");
			$fd->print("\tprintf\"OSRTE_OS  =%s\n\", $OSRTE{'OS'})\n");
			$fd->print("\tprintf\"OSRTE_ARCH=%s\n\", $OSRTE{'ARCH'})\n");
			$fd->print("\tprintf\"OSRTE_CC  =%s\n\", $OSRTE{'CC'})\n");
			$fd->print("\tprintf\"OSRTE_CPP =%s\n\", $OSRTE{'CPP'})\n");
			$fd->print("}\n");
            $fd->close;

            {
                local  @ARGV = ( @Flags, $BuildModule );
                do "$ENV{TOOL}/bin/compc$ENV{TOOLEXT}";
                if ( $@ ) { die "$@"; exit }
            }
        }
        return( $BuildObj );
    }
    else
    {
        return ();
    }
}

###############################################################

sub GetBuildNrModule
{
    return( $BuildNrFunction."_".$ENV{BUILDPID} );
}

###############################################################

sub SetBuildNrModule
{
    local $func = shift;
    if ( defined($func) )
    {
        $BuildNrFunction = $func;
    }
    else
    {
        undef $BuildNrFunction;
    }
}

###############################################################

sub CheckIncludes
{
    my( $r_Includes, $File, $LAYER ) = @_;

    local $fd = new FileHandle;
    if ( ! $fd->open($File, "r") )
    {
        warn "Can't open $File (input) :$!\n";
    }
    else
    {
        # check includes in $SOURCE with %INCLUDES
        my ($incname, $inc, $slowlevel, $lookup, %INCLUDES);
        $lookup = "";
        $lookup = "(QUICK_MSP00|SLOW_MSP00|QUICK_MEO00|SLOW_MEO00)" if $ENV{'CURR_VERSION'} =~ /^f/i;
        $lookup = "(SLOW_MSP00|SLOW_MEO00)" if $ENV{'CURR_VERSION'} =~ /^q/i;
        my $wrk = $ENV{'WRK'};
        $wrk =~ tr[\\][/];
        foreach $inc ( @$r_Includes )
        {
            $inc =~ tr[\\][/];
            my $ind = index($inc, "$wrk/incl/");
            if ( $ind == 0 )
            {
                $incname = substr( $inc, length("$wrk/incl/") );
                ( $incname ) = $incname =~ m![/]?(.+)!;
            }
            else { $incname = $inc }
			if ( $incname =~ /^(.*)(\(noextension\)|\.)$/)
			{	$INCLUDES{$1} = "";}
			else
			{
				$incname .= ".h" unless $incname =~ /\.(h|hpp|c|cpp)$/;
				$INCLUDES{$incname} = "";
			}
        }
        local $maxlines = 1200;
        while ( <$fd> ) {
            last if $. > $maxlines;

            if ( ($ENV{'CURR_VERSION'} !~ /^s/i) &&
                 /#\s*if\s+(COMPILEMODE_MSP00|COMPILEMODE_MEO00).+$lookup/ )
            {
                while ( !/#\s*endif/ and !eof ) { $_ = <$fd> }
            }

			# CTS 1107626
			if ( /^\s*\/\*\s* set maxlines\s*=\s*(\d*)\s*\*\//i )
	        {
				$maxlines = $1;
			}

            #   if ( /^\s*#\s*include\s*"(.*)"(?!.*no\s*check)/ )
		    if ( /^\s*#\s*include\s*"(.*)"(.*)$/ )
            {
				$incname = $1;
				$is_nocheck = ( $2 =~ /.*no\s*check/ );

			    if ( defined( $INCLUDES{$incname} ) )
                {
					$INCLUDES{$incname} = "1";
				}
				elsif ( defined( $INCLUDES{"SAPDB/".$incname} ) )  # CTS 1105037
				{
					$INCLUDES{"SAPDB/".$incname} = "1";
                }
				# for STL (not right description implementation of STL) ->
				elsif ( defined( $INCLUDES{"SAPDB/STLport/stlport/".$incname} ) )
				{
					$INCLUDES{"SAPDB/STLport/stlport/".$incname} = "1";
                }
				elsif ( ($LAYER ne "") && defined( $INCLUDES{"$LAYER/$incname"} ))
				{
					$INCLUDES{"$LAYER/$incname"}="1";
				}
				elsif ( ($LAYER ne "") && defined( $INCLUDES{"$WRK/incl/$LAYER/$incname"} ))
				{
					$INCLUDES{"$WRK/incl/$LAYER/$incname"}="1";
				}
                else
                {
                    warn "Include $incname not in include list for $File\n($maxlines lines examined)\n"
						unless ($is_nocheck);
                }
			}
        }
        $fd->close;
        foreach $key ( keys %INCLUDES )
        {
            warn "$key in include list for $File but not included\n($maxlines lines examined)\n"
                if $INCLUDES{$key} ne "1";
        }
    }
}

###############################################################

sub ToolTrace
{
	# usage ToolTrace($string, \@daten)
    $TraceFile = "$WRK/$VMAKE_VERSION/prot/trace.prot";
	if ( $ENV{'TOOLTRACE'})
    {
		local( $string, $r_daten ) = @_;
		open( TOOLTRACE, ">>$TraceFile") || die "Can't open $TraceFile (append): $!\n";
		print TOOLTRACE sprintf($string, @$r_daten);
		close(TOOLTRACE);
	}
}

###############################################################

sub ProtokollName
{
    my $PROG = shift;

    my $fh = new FileHandle "dummy.dbcmd", "w";
    #open(FILE_OUT, "+>dummy.dbcmd");
    #print FILE_OUT "vdnexit!\n";
    #close FILE_OUT;
    $fh->print("vdnexit!\n");
    $fh->close;

    system("$PROG -b dummy.dbcmd >$NULL");

    my $CMD = "ls -1tr *.prot";
    my $CI_PROT;
    open(PIPE_IN, "$CMD |");
    while(<PIPE_IN>) { chomp; $CI_PROT = $_ }
    close PIPE_IN;

    unlink("dummy.dbcmd");
    return $CI_PROT;
}

###############################################################

sub Timestamp
{
    my $no_linefeed = shift;
    local($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = localtime(time);
	$year = $year - 100 if ( $year > 99 ); # PTS 1105199
	$mon++; # adapt month
    foreach ( "mon", "mday", "hour", "min", "sec" )
    {
	    $$_ = "0$$_" if length($$_) == 1;
    }
    my $timestamp = "$year-$mon-$mday-$hour.$min.$sec";
	$timestamp="0".$timestamp if ( $year < 10 );  # PTS 1105199
    $timestamp .= "\n" unless $no_linefeed;
    return($timestamp);
}

##################################################################

sub GetConnectLevel
{
    my $fh;
    local $rc;
    if ( $fh = new FileHandle "CONNECT.cmd", "r" )
    {
        while(<$fh>)
        {
            if ( /^CONNECT &1 IDENTIFIED BY &2 &3 &4 ISOLATION LEVEL (\d+).*/i )
            {
                $rc = $1; last;
            }
        }
        $fh->close;
    }
    return $rc;
}

##################################################################

sub GetProfilePath
{

    my $Release   = shift;
    my $ProfileDB = "\\\\P26326\\ToolSrc\\profiles";
    my %ProfileDB;
    my $ProfilePath;

    dbmopen(%ProfileDB, $ProfileDB, 0) ||
	warn "Can't open Profile-Database!: $!\n";

    $ProfilePath = %ProfileDB->{$Release};

    dbmclose(%ProfileDB);

    return($ProfilePath);

}

##################################################################

sub GetAllProfiles
{

    my $ProfileDB = "\\\\P26326\\ToolSrc\\profiles";
    my %ProfileDB;
    my @ProfileKeys;

    if ( dbmopen(%ProfileDB, $ProfileDB, 0) ) {

	@ProfileKeys = keys %ProfileDB;

	dbmclose(%ProfileDB);
    }

    return (@ProfileKeys);
}

########################################################################
# &gar  (CTS 1108093)                                                  #
########################################################################
# Funktion: SrcCopy($source, $target)
#
# src_copy kopiert eine Datei in ein Verzeichnis und hebt,
# wenn vorhanden, die Schreibsperre auf.
#
# $source  : absoluter Dateipfad
# $target  : absoluter Zielpfad
# Rckgabe : 1 bei Erfolg, 0 sonst, $! wird gesetzt
#
#########################################################################
sub SrcCopy
{
	local($source, $target) = @_;
	local $rc;

	if (-f $target)
	{
		chmod 0664, "$target";
		unlink ($target);
	}

	$rc = File::Copy::copy($source, $target) ;
	return 0 unless ( $rc );

     chmod 0664, "$target" if (-f $target);

	return 1;
}
