
#
#    ========== licence begin LGPL
#    Copyright (C) 2002 SAP AG
#
#    This library is free software; you can redistribute it and/or
#    modify it under the terms of the GNU Lesser General Public
#    License as published by the Free Software Foundation; either
#    version 2.1 of the License, or (at your option) any later version.
#
#    This library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public
#    License along with this library; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#    ========== licence end
#

import string
import re
import os
import shutil

import optlib

#
# files to exclude
#

_passwordProc = [
    # 'sys/src/sp/vsp02',
    # 'sys/src/sp/vsp02.c',
    # 'sys/src/sp/vsp02.cpp',
    # 'sys/src/jv/com/sap/dbtech/jdbc/NameHandling.java',
    # 'sys/src/jv/sapdbc.src.jar',
]

_sapObj = [
    'sys/src/co/sap/sapobj/',
    'sys/src/sap/sapobj/',
    'sys/src/sap45A/',
    'sys/src/sap50A/',
]

_obsolete = [
    'sys/src/qu/',
]

_testing = [
    'sys/src/pf/',
    'sys/src/jtdev/',
    'sys/src/jtest/',
    'sys/src/su/',
]

_buildObsolete = [
    'sys/src/SAPDB/PTS',
    'sys/src/tt/mftools',
    'sys/src/tt/vmake',
]

_excludeFiles = _passwordProc + _sapObj + _obsolete + _testing + _buildObsolete

def isExcluded (fname, patterns = _excludeFiles):
    for pattern in patterns:
        if fname [:len (pattern)] == pattern:
            return 1
    return None


#
# command line options
#

import perforce.perforceLib

_cmdlineOptions = [
        ('ftp', None, ':', 'FileExchange:ExchangeFile@p48047', 'ftp login'),
        ('staging', None, ':', '/staging/sapdb', 'staging dir on FTP server'),
        ('p4', 'perforceClient', ':', None, 'name of perforce client'),
        ('root', None, '::', None, 'root dir in depot notation'),
        ('perforceRoot', None, ':', None, 'use all directories under this root'),
        ('o', 'outdir', ':', '.', 'output directory'),
]

def checkOptions (options):
    if not options.perforceClient:
        optlib.fatalError ('perforce client must be specified (-p4)')
    options.p4 = perforce.perforceLib.PerforceSession (options.perforceClient)
    options.root = getRoots (options)
    assert options.root, '-root must be specified'
    for root in options.root:
        if root [:2] != '//':
            optlib.fatalError ('-root not in perforce depot notation\n')
    options.outdir = os.path.abspath (options.outdir)

def getRoots (options):
    if options.perforceRoot:
        lines = options.p4.getLines (' dirs ' + options.perforceRoot + '/*')
        return map (string.strip, lines)
    else:
        return options.root

#
# copy standard source files
#
_licenceFiles = [
    'licence/gpl.txt', 'licence/lesser.txt', 'licence/fdl.txt',
]

_standardFiles = _licenceFiles [:]

def copyStandardFiles (targetDir, readmes = []):
    candidates = [
        # os.path.join (os.environ ['SRC'], 'data'),
        os.path.join (os.environ ['TOOL'], 'data')]
    result = []
    for file in _standardFiles:
        fullSource = findFile (file, candidates)
        if not fullSource:
            continue
        fullTarget = os.path.join (targetDir, file)
        fullDir = os.path.dirname (fullTarget)
        if not os.path.isdir (fullDir):
            os.makedirs (fullDir)
        shutil.copy (fullSource, fullTarget)
        result.append (file)
    candidates = [
        os.path.join (os.environ ['SRC'], 'data'),
        os.path.join (os.environ ['TOOL'], 'data')]
    for inname, outname in readmes:
        readmeIn = findFile (inname, candidates)
        if readmeIn:
            shutil.copy (readmeIn, os.path.join (targetDir, outname))
        result.append (outname)
    return result

def findFile (fname, dirs):
    for dir in dirs:
        fullPath = os.path.join (dir, fname)
        print 'trying', fullPath
        if os.path.exists (fullPath):
            return fullPath
    return None

#
# ftp
#

import ftplib

def transferByFTP (options, fname, subdir = '/7.3/updates'):
    """transfer a file to the FTP staging area
    """
    if not options.ftp:
        return
    match = re.match ('(.*):(.*)@(.*)', options.ftp)
    user, password, host = match.group (1, 2, 3)
    session = ftplib.FTP (host, user, password)
    try:
        remoteDir = options.staging + subdir
        session.cwd (remoteDir)
        remoteName = os.path.basename (fname)
        stream = open (fname, 'rb')
        session.storbinary ('STOR ' + remoteName, stream, 4096)
    finally:
        session.quit ()


